# -*- coding: utf-8 -*

#-------------------------------------------------------------------------------
# Name:        module1
# Purpose:
#
# Author:      Johannes Bichler 
#
# Created:     08.08.2021
# Copyright:   (c) Johannes Bichler 2021
# Licence:     <your licence>
#-------------------------------------------------------------------------------

#from __future__ import print_function
#from __future__ import division

import sys
#sys.path.append('/home/jb/miniconda3/envs/py369/lib/python3.6/site-packages')

#from builtins import str
#from builtins import range
#from builtins import object
from FreeCAD import Base,Console
import FreeCAD
import MPbase
import re
import Part
import time
import numpy as np


#########################################################
#########################################################


def calculatePlacement(refbase,relfLCS,vecs,pt,show_LKS_systems):
	'''
	usually:
	refbase=FreeCAD.Vector(0,0,0)
	relfLCS=[FreeCAD.Vector(1,0,0),FreeCAD.Vector(0,1,0),FreeCAD.Vector(0,0,1)] 
	
	hardcoded for gun origin at Vector(0,0,0) without arc_length
	orientation vector along x axis
	'''
	#calcuate Angles
	#vec[0] normal vector to section
	#vec[1] x-axis of weld
	#vec[2] y-axis of weld

	import FreeCAD

	base=refbase
	x_axis=relfLCS[0] #positive travel direction of weld head
	y_axis=relfLCS[1]
	z_axis=relfLCS[2] #positive direction from weld head tip to robot mounting (weld wire axis)
	
	rot_a=FreeCAD.Rotation(vecs[0],x_axis)
	plc_0=FreeCAD.Placement(base,rot_a)
	vecs_2=plc_0.multVec(vecs[2])
	#print vecs_2
	
	#ang=math.degrees(vecs_2.getAngle(z_axis))
	#rot_b=FreeCAD.Rotation(x_axis,ang)
	rot_b=FreeCAD.Rotation(vecs_2,z_axis)
	inter_plc=FreeCAD.Placement(base,rot_b)
	#print(rot_b)
	#print(inter_plc)
	plc_1=inter_plc.multiply(plc_0)
	vecs_3=inter_plc.multVec(vecs_2)
	#print(vecs_3)
	
	
	plc_1=plc_1.inverse()
	plc_1.move(pt)
	plc_0.move(pt)
	inter_plc.move(pt)
	if (show_LKS_systems & 0b0001)==0b0001:
		#draw a coordinate system
		x=Part.makeLine(base,x_axis*100)
		y=Part.makeLine(base,y_axis*200)
		z=Part.makeLine(base,z_axis*400)
		weld_head=Part.Compound([x,y,z])
		weld_head.Placement=plc_1
		#Console.PrintMessage("++++LKS positioner point:  " + str(positioned_poi) + '\n')
		myObject=FreeCAD.ActiveDocument.addObject("Part::Feature","Weld head")
		myObject.Shape = weld_head
	ABC=plc_1.Rotation.toEuler()
	A=round(ABC[0],2)
	B=round(ABC[1],2)
	C=round(ABC[2],2)
	return plc_0,inter_plc,plc_1

#########################################################
#########################################################


class SmlData(object):
	'''
	should extract/be in sync with Structure()
	'''
	def __init__(self):

		self.currLayer="Y:/Client_0/Weld_0/Version_0/Pattern_0/Scan_0/Layer_0/" #gets overwritten by setDirs

		self.Pathsettingsfile='pathsettings.txt'
		self.Pathclassrobot='wrtrnb.txt'
		self.Pathfaces='pathfaces.stp'
		self.Pathborders='pathborders.stp'
		self.Pathcurve='pathcurve.stp'
		self.Pathdatafile='pathdata.txt'
		self.Pathcodeweld='pathcodeweld.csv'
		self.Pathcoderapid='pathcoderapid.csv'

#########################################################
#########################################################

def loadPathfaces(c_data):
	'''
	search for all pathfaces in drive 
	specified in c_data
	load them form file and display in correct order
	return the sorted paths by robotnb
	'''	
	final_files=MPbase.filesearch2(c_data.currLayer,c_data.Pathfaces[:-4]+'*',len(c_data.Pathfaces[:-4]),-4)
	
	print(final_files)
	cr_int=[]
	rob=[]
	robpnb=[]
	figures=[]
	#cr_names=[]
	regex = r'\d+'	#find numbers
	
	div=float(len(final_files))
	print('Final files: ',len(final_files))
	base=1/(div-1) #float /255.
	print(base)
	
	colors=[]
	
	
	pathsfigure=restdata(c_data.currLayer,c_data.Pathcurve)
	pathsborder=restdata(c_data.currLayer,c_data.Pathborders)
	
	for j,ff in enumerate(final_files):
		match = re.findall(regex, ff)
		match=[int(m) for m in match]
		rbnb,pnb,core=match
		if rob.count(rbnb)==0:
			rob.append(rbnb)
			robpnb.append([])
			figures.append([])
		idx=rob.index(rbnb)
		robpnb[idx].append(pnb)
		
		comp=[]
		pathfaces=Part.Shape()
		pathfaces.read(c_data.currLayer+ff)
		comp.append(pathfaces)
		
		pf_idx=pathsfigure[0].index(rbnb)
		if pathsfigure[1][pf_idx].count(pnb)>0:
			file0_idx=pathsfigure[1][pf_idx].index(pnb)
			figshape=Part.Shape()
			figshape.read(c_data.currLayer+pathsfigure[2][pf_idx][file0_idx])
			comp.append(figshape)
		
		pb_idx=pathsborder[0].index(rbnb)
		if pathsborder[1][pb_idx].count(pnb)>0:
			file1_idx=pathsborder[1][pb_idx].index(pnb)
			bordershape=Part.Shape()
			bordershape.read(c_data.currLayer+pathsborder[2][pb_idx][file1_idx])
			comp.append(bordershape)
		
		viewdata=Part.Compound(comp)
		
		name=c_data.Pathfaces[:-4]+'_'+str(rbnb)+'_'+str(pnb)
		figures[idx].append([name,viewdata])
		
		color=(j*base,j*base,1.0-j*base)
		print('color: ',color)
		colors.append(color)
		cr_int.append([int(m) for m in match]) #idxes
	
	cnt=0
	ordered=[]
	for i in range(len(rob)):
		curr_block=robpnb[i][:]
		curr_block.sort()
		ordered.append([])
		for actual in curr_block:
			print('Actual: ',actual)
			idx=robpnb[i].index(actual)
			data=figures[i][idx]
			color=colors[cnt]
			ordered[-1].append([data[1],data[0],color])	
			'''		
			myObject=App.ActiveDocument.addObject("Part::Feature",data[0])
			myObject.ViewObject.Transparency=40 #0..100
			myObject.ViewObject.ShapeColor=colors[cnt]
			myObject.Shape = data[1]
			'''
			cnt=cnt+1
	
	return ordered,curr_block

#########################################################
#########################################################

def colormap(nbelems,start,finish):
	div=float(nbelems-1)
	base=1/(div-1) #/255.
	delta=[((finish[i]-start[i])/div) for i in range(len(start))]
	colors=[]
	shuffle=[]
	for j in range(nbelems):
		#color=(j*base,j*base,1.0-j*base)
		color=(start[0]+j*delta[0],start[1]+j*delta[1],start[2]+j*delta[2])
		print('color: ',color)
		colors.append(color)
	
	for i in range(nbelems):
		comp_i=nbelems-i
		print(comp_i)
		if i%2==0:
			n=i
		else:
			n=comp_i
		shuffle.append(colors[n])
	
	return colors,shuffle
		
#########################################################
#########################################################


def restdata(dir,filename):
	final_files=MPbase.filesearch2(dir,filename[:-4]+'*',len(filename[:-4]),-4)
	cr_int=[]
	rob=[]
	robpnb=[]
	figures=[]
	#cr_names=[]
	regex = r'\d+'	#find numbers
	
	grouped_files=[]	
	for j,ff in enumerate(final_files):
		match = re.findall(regex, ff)
		match=[int(m) for m in match]
		rbnb,pnb,core=match
		if rob.count(rbnb)==0:
			rob.append(rbnb)
			robpnb.append([])
			figures.append([])
			grouped_files.append([])
		idx=rob.index(rbnb)
		robpnb[idx].append(pnb)
		grouped_files[idx].append(ff)	
	
	return rob,robpnb,grouped_files


#########################################################
#########################################################


def showPaths(c_data):
    crx,b=loadPathfaces(c_data)

    #interpolate between colors
    #sortaf,af=colormap(len(crx[0]),(1.0,0.5,0.0),(0.0,0.5,1.0))

    af=[(0.00,0.00,1.00),(0.67,1.00,0.00),(1.00,1.00,0.00),(1.00,0.00,0.00)] #color codes
    cnt=0
    for data in crx:
        for dat in data:
            #FreeCAD.Gui.updateGui()
            myObject=FreeCAD.ActiveDocument.addObject("Part::Feature",dat[1])
            #Gui.Selection.addSelection(myObject)
            myObject.ViewObject.Transparency=0 #0..100
            myObject.ViewObject.ShapeColor=(170/255.,1.0,0.)
            myObject.ViewObject.LineColor=(170/255.,1.0,0.)
            myObject.Shape = dat[0]
            FreeCAD.Gui.updateGui()
            time.sleep(0.1)
            myObject.ViewObject.ShapeColor=af[cnt] #dat[2]
            myObject.ViewObject.LineColor=(0.0,0.0,0.0) #af[cnt] #dat[2]
            myObject.ViewObject.PointColor=(0.0,0.0,0.0) #af[cnt] #dat[2]
            myObject.ViewObject.Transparency=0 #0..100
            #Gui.Selection.removeSelection(myObject)
            cnt=cnt+1
            if cnt>=len(af):
                cnt=0
    return None

#########################################################
#########################################################

def extractpaths(c_data):
	weld_files=MPbase.filesearch2(c_data.currLayer,c_data.Pathcodeweld[:-4]+'*',len(c_data.Pathcodeweld[:-4]),-4)
	weld_nbs=MPbase.extractnumbers(weld_files)
	rapid_files=MPbase.filesearch2(c_data.currLayer,c_data.Pathcoderapid[:-4]+'*',len(c_data.Pathcoderapid[:-4]),-4)
	rapid_nbs=MPbase.extractnumbers(rapid_files)
	
	#wns=weld_nbs[1][:]
	#rns=weld_nbs[1][:]
	
	np_wns=np.asarray(weld_nbs)
	np_wns=np_wns.transpose()
	wns_order=np.lexsort([np_wns[:,1],np_wns[:,0]])
	
	np_rns=np.asarray(rapid_nbs)
	np_rns=np_rns.transpose()
	rns_order=np.lexsort([np_rns[:,1],np_rns[:,0]])
	
	arrangement=[]
	for i in range(wns_order.shape[0]):
		if i<wns_order.shape[0]:
			color=(170/255,255/255,0.0)
			arrangement.append([c_data.currLayer+weld_files[wns_order[i]],"PathSeg",color])
		if i<rns_order.shape[0]:
			color=(1.0,0.0,0.0)
			arrangement.append([c_data.currLayer+rapid_files[rns_order[i]],"Rapid",color])
		
	blockelem=[]
	for singleelem in arrangement:
		
		data = np.genfromtxt(singleelem[0], dtype=None , delimiter="\t", missing_values=True,filling_values=True,skip_header=0, names=True) #,encoding=None) #, missing_values='',filling_values='',encoding='bytes')
		#print(data)
		
		fields=['x','y','z','ax','ay','az','bx','by','bz','cx','cy','cz']
		idxs=[]
		for f in fields:		
			n_idx=data.dtype.names.index(f)
			idxs.append(n_idx)
		
		nidxs=[]
		nonfields=['Func']
		for f in nonfields:		
			n_idx=data.dtype.names.index(f)
			nidxs.append(n_idx)
		
		np_row=[]
		for d in data:
			rowlist=list(d)
			if rowlist[nidxs[0]].decode()=='@':
				x=np.asarray([rowlist[idx] for idx in idxs])
				x=x.reshape(1,-1)
				np_row.append(x)
		
		waypts=np.concatenate(np_row,axis=0)
			
		source_pts=[Base.Vector(w) for w in waypts[:,0:3]]
		sourceLCS=[[Base.Vector(w[0:3]),Base.Vector(w[3:6]),Base.Vector(w[6:9])] for w in waypts[:,3:]]
		ident=singleelem[1]
		color=singleelem[2]	
		
		blockelem.append([source_pts,sourceLCS,ident,color])
	return blockelem

######################################################################
######################################################################

def createpath(blockelem,showrapid,showseg):
	places=[]
	refbase=Base.Vector(0,0,0)
	refLCS=[Base.Vector(1,0,0),Base.Vector(0,1,0),Base.Vector(0,0,1)] #positive travel direction of weld head
	for j,block in enumerate(blockelem):
		source_pts=block[0] #vis_pts #p_pts
		sourceLCS=block[1]#nwLCS #pLCS
		
		if block[2]=="Rapid" and showrapid is True:
			pathwire=Part.makePolygon(source_pts)
			myObject=FreeCAD.ActiveDocument.addObject("Part::Feature",block[2])
			color=(1.0,0.0,0.0)
			myObject.ViewObject.LineColor=block[3]
			pathwire=Part.makePolygon(source_pts)
			myObject.Shape = pathwire
		elif block[2]=="PathSeg" and showseg is True:
			myObject=FreeCAD.ActiveDocument.addObject("Part::Feature",block[2])
			color=(1.0,0.0,0.0)
			myObject.ViewObject.LineColor=block[3]
			pathwire=Part.makePolygon(source_pts)
			myObject.Shape = pathwire
		
		for i in range(len(sourceLCS)):	#range(len(newLCS))
			nb=0
			vec_2=sourceLCS[i][0].cross(sourceLCS[i][1])
			wpts=[source_pts[i]]
			LKS_vecs=[[sourceLCS[i][0],sourceLCS[i][1],sourceLCS[i][2]]]
			new_place=calculatePlacement(refbase,refLCS,LKS_vecs[nb],wpts[nb],0b0000)
			places.append(new_place[2])
			
	return places

######################################################################
######################################################################

def simpath(toolobj,ref_place,places,targetidx):
	new_abs_place=places[targetidx].multiply(ref_place)
	toolobj.Placement=(new_abs_place)
	FreeCAD.Gui.updateGui()

######################################################################
######################################################################



