# FreeCAD TemplatePyMod module  
# (c) 2011 Werner Mayer LGPL


#from FreeCAD import Base,Part
import FreeCADGui
from PySide import QtGui,QtCore

import yaml
from pivy import coin

import sys
#sys.path.append( "/home/jb/pgr3")
#sys.path.append( "/home/jb/pgr3/weldslicerWB")
 
from FreeCAD import Console,Base
import FreeCAD,Part
import STLfuncs
import STLvis
import wc
import time
import os

import pathlib
#FreeCADGui.PySideUic.loadUi(str(pathlib.Path(__file__).with_suffix(".ui")))
basefolder=pathlib.Path(__file__).parent
#basefolder="/home/jb/pgr3/weldslicerWB"

##############################################################################
##############################################################################

class seclist():
	def __init__(self):
		self.FaceComp=None
		self.Wire=None
		self.Edge=None
		self.StartPt=None
		self.StartVertex=None
		self.Pts=None
		self.Vecs=None
		self.closed=False

##############################################################################
##############################################################################


class CmdFacePanel():
	def GetResources(self):
		return {'Pixmap'  : os.path.join(basefolder,"Resources","1.svg"),
				'Accel' : "", 
				'MenuText': "Faceselection",
				'ToolTip' : "Get the faces to extract wires and the base vectors of the weld"}
	
	def Activated(self):
		"""Do something here"""
		panel = FacePanel()
		FreeCADGui.Control.showDialog(panel)
		return		
	
	def IsActive(self):
		"""Here you can define if the command must be active or not (greyed) if certain conditions
		are met or not. This function is optional."""
		return True

##############################################################################
##############################################################################

class CmdWireTaskPanel():
 
	def GetResources(self):
		'''
		return {'Pixmap'  : os.path.join(basefolder,"Resources","2.svg"),
				'Accel' : "Shift+S", 
				'MenuText': "Select connected edges/wires ",
				'ToolTip' : "Create the wire to slice along"}
		'''
		return {'Pixmap'  : os.path.join(basefolder,"Resources","2.svg"),
				'Accel' : "", 
				'MenuText': "Select connected edges/wires ",
				'ToolTip' : "Create the wire to slice along"}
	
	def Activated(self):
		"""Do something here"""
		panel = WireTaskPanel()
		FreeCADGui.Control.showDialog(panel)
		return

	def IsActive(self):
		"""Here you can define if the command must be active or not (greyed) if certain conditions
		are met or not. This function is optional."""
		if obj.FaceComp is not None:
			return True
		else:
			return False


##############################################################################
##############################################################################

class CmdPointTaskPanel():

	def GetResources(self):
		return {'Pixmap'  : os.path.join(basefolder,"Resources","3.svg"),
				'Accel' : "", 
				'MenuText': "Wireselection",
				'ToolTip' : "Discretize the selected weldcurve (wire) into points"}
	
	def Activated(self):
		"""Do something here"""
		panel = PointTaskPanel()
		FreeCADGui.Control.showDialog(panel)
		return

	def IsActive(self):
		"""Here you can define if the command must be active or not (greyed) if certain conditions
		are met or not. This function is optional."""
		if obj.Wire is not None:
			return True
		else:
			return False

##############################################################################
##############################################################################

class CmdWpsTaskPanel():
	
	def GetResources(self):
		return {'Pixmap'  : os.path.join(basefolder,"Resources","4.svg"),
				'Accel' : "", 
				'MenuText': "Sequencecreation",
				'ToolTip' : "Create the sequences and export input file for weldslicer"}
	
	def Activated(self):
		"""Do something here"""
		panel = WpsTaskPanel()
		FreeCADGui.Control.showDialog(panel)
		return

	def IsActive(self):
		"""Here you can define if the command must be active or not (greyed) if certain conditions
		are met or not. This function is optional."""
		if obj.Pts is not None:
			return True
		else:
			return False

##############################################################################
##############################################################################
 
class CmdConnectedFacesTaskPanel():
	
	def GetResources(self):
		return {'Pixmap'  : os.path.join(basefolder,"Resources","5.svg"),
				'Accel' : "", 
				'MenuText': "Get weld groove faces",
				'ToolTip' : "Get the weld groove faces and export .stl"}
	
	def Activated(self):
		"""Do something here"""
		panel = ConnectedFacesTaskPanel()
		FreeCADGui.Control.showDialog(panel)
		return
		
##############################################################################
##############################################################################

class CmdWeldCoverFacesTaskPanel():
 	
	def GetResources(self):
		return {'Pixmap'  : os.path.join(basefolder,"Resources","6.svg"),
				'Accel' : "", 
				'MenuText': "Get weld top cover face",
				'ToolTip' : "Get weld top cover face and export .stl"}	
	
	def Activated(self):
		"""Do something here"""
		panel = WeldCoverFacesTaskPanel()
		FreeCADGui.Control.showDialog(panel)
		return

##############################################################################
##############################################################################

class CmdshowPathsTaskPanel():
 	
	def GetResources(self):
		return {'Pixmap'  : os.path.join(basefolder,"Resources","7.svg"),
				'Accel' : "", 
				'MenuText': "Show the paths from choosen folder",
				'ToolTip' : "Import .stp and show paths"}	
	
	def Activated(self):
		"""Do something here"""
		inst=showPathsTaskPanel()
		inst.load()
		#panel = showPathsTaskPanel()
		#FreeCADGui.Control.showDialog(panel)
		return

##############################################################################
##############################################################################

class CmdsimTaskPanel():
 	
	def GetResources(self):
		return {'Pixmap'  : os.path.join(basefolder,"Resources","8.svg"),
				'Accel' : "", 
				'MenuText': "Simulate the paths from choosen folder",
				'ToolTip' : "Import weld and rapid paths from .csv and simulate them"}	
	
	def Activated(self):
		"""Do something here"""
		panel = SimTaskPanel()
		FreeCADGui.Control.showDialog(panel)
		#inst=SimTaskPanel()
		#inst.load()
		return

##############################################################################
##############################################################################


class FacePanel():
	
	def __init__(self):
		# this will create a Qt widget from our ui file
		path_to_ui =os.path.join(basefolder,"faceselection.ui")
		self.form = FreeCADGui.PySideUic.loadUi(path_to_ui)
	
	def accept(self): #accept
		bodys,elx,objlist=fromSelection()
		print ('elx: ',elx)
		facecomp=Part.Compound(elx)
		obj.FaceComp=facecomp
		wires=STLfuncs.Basecurve(facecomp)
		
		for wire in wires:
			#Part.show(wire)
			color=(170/255.,170/255.,127/255.)
			myObject=FreeCAD.ActiveDocument.addObject("Part::Feature","Basecurve")
			
			myObject.ViewObject.LineColor=color
			myObject.Shape = wire
		
		FreeCADGui.Control.closeDialog()

##############################################################################
##############################################################################
 
class WireTaskPanel:
	#create a wire out of edges or other wires
	def __init__(self):
		# this will create a Qt widget from our ui file
		path_to_ui =os.path.join(basefolder,"wireselection.ui")
		self.form = FreeCADGui.PySideUic.loadUi(path_to_ui)
	
	def accept(self):
		'''
		sf = FreeCADGui.Selection.getSelectionEx()
		print(sf)
		elems=[]
		for sel in sf:
			if sel.HasSubObjects is False:
				selelm=sel.Object.Shape.Edges
			else:
				selelm=sel.SubObjects
			elems.extend(selelm)
		'''
		
		bodys,grouped,objlist=fromSelection()		
		#sx = self.form.sel()
		elems=[]
		for f in grouped:
			#Part.show(f)
			print(f)
			elems.extend(f.Edges)
		
		wirecomp=Part.Wire(Part.__sortEdges__(elems))
		obj.Wire=wirecomp
		#wires=STLfuncs.Basecurve(facecomp)
		
		#for wire in wires:
		#Part.show(wire)
		color=(170/255.,170/255.,127/255.)
		myObject=FreeCAD.ActiveDocument.addObject("Part::Feature","Weldcurve")
		myObject.ViewObject.LineColor=color
		myObject.Shape = wirecomp
		
		FreeCADGui.Control.closeDialog()


##############################################################################
##############################################################################
 
class PointTaskPanel:
	#select the basecurve, startvertex and startedge and create the the pts and vecs list
	def __init__(self):
		# this will create a Qt widget from our ui file
		path_to_ui =os.path.join(basefolder,"ptscreation.ui")
		self.form = FreeCADGui.PySideUic.loadUi(path_to_ui)
		
	def accept(self):
		sf = FreeCADGui.Selection.getSelectionEx()
		print(sf)
		elems=[]
		for sel in sf:
			elems.extend(sel.SubObjects)
		
		edge=sf[0].SubObjects[0]
		startvertex=sf[0].SubObjects[1]
		obj.Edge=edge
		obj.StartVertex=startvertex
		obj.StartPt=startvertex.Point
		
		#wire=sf[0].Object.Shape
		wire=obj.Wire
		
		start_processing=time.time()
		y,y_direction=STLfuncs.wiresort_smart(wire,obj.Edge,obj.StartPt) #wire_0
		end_processing=time.time()
		Console.PrintMessage("Overall processing:  " + str(round(end_processing-start_processing,3)) + " [sec]"+'\n')
		base_wire=Part.Wire(y)
		
		if base_wire.isClosed() is True:
			obj.closed=True
		#new method
		start_processing=time.time()
		#pts=section_pts2(base_wire,y_direction,edge,startpoint,0.1,0.01,'AngularCurvature')
		pts=STLfuncs.section_pts4(base_wire,edge,startvertex,0.1,0.01,'AngularCurvature')
		pts=pts[0]
		#CLEAN THE POINTS FROM DUPLICATES (necessary to prevent path creation from failing)

		pts=STLfuncs.RemoveDuplicatePts(pts,1e-3)

		'''
		u=wc.sumVecLengths(pts,False)
		last=0
		npts=[]
		for i,x in enumerate(u[0]):
			if x<last+0.1 and i!=0:
				print(i)
			else:
				npts.append(pts[i])
			last=x
		
		pts=npts
		'''
		
		obj.Pts=pts		
		ptsobj=Part.Compound([Part.Vertex(p) for p in pts])
		
		#for wire in wires:
		#Part.show(wire)
		color=(170/255.,170/255.,127/255.)
		myObject=FreeCAD.ActiveDocument.addObject("Part::Feature","Ptsrep")
		myObject.ViewObject.LineColor=color
		myObject.Shape = ptsobj
		
		start_processing=time.time()
		vec_list=STLfuncs.weld_base_model_values_point_list2(obj.FaceComp.Faces,base_wire,y_direction,obj.Pts)
		obj.Vecs=vec_list
		#vec_list=STLfuncs.weld_base_model_values_point_list2(facecomp.Faces,base_wire,y_direction,pts)
		end_processing=time.time()
		Console.PrintMessage("Vec list:  " + str(round(end_processing-start_processing,5)) + " [sec]"+'\n')
		
		ptnnb=[i for i in range(len(pts))]
		baselength=10
		if obj.closed is True:
			markers=[ptnnb[0],ptnnb[max(ptnnb)//4*1],ptnnb[max(ptnnb)//4*2],ptnnb[max(ptnnb)//4*3]] #,ptnnb[-1]]
		else:
			markers=[ptnnb[0],ptnnb[max(ptnnb)//3*1],ptnnb[max(ptnnb)//3*2],ptnnb[max(ptnnb)]] #,ptnnb[-1]]			
		for i in (markers):
			origin=STLfuncs.show_vecs(pts,vec_list,baselength,[i])
			myObject=FreeCAD.ActiveDocument.addObject("Part::Feature","Vecs_"+str(i))
			myObject.Shape = origin
		
		#STLfuncs.slicerCSV(obj.Pts,obj.Vecs,"/home/jb/pgr3/weldslicerWB/slicepts.csv")
		
		FreeCADGui.Control.closeDialog()


##############################################################################
##############################################################################

class WpsTaskPanel:
	def __init__(self):
		with open(os.path.join(basefolder,"wpsdemo.wps"), 'rt',encoding='ISO-8859-1') as ymlfile:
			cfg = yaml.safe_load_all(ymlfile) #only at version yaml 5.1
			self.cfg = list(cfg)[0]
		path_to_ui =os.path.join(basefolder,"wps.ui")
		self.form = FreeCADGui.PySideUic.loadUi(path_to_ui)
		self.form.setWindowTitle(
			QtGui.QApplication.translate("WeldslicerAddon", "Slice Points", None)
		)

		self.form.scrollbarptnb.valueChanged.connect(self.valuechange)
		self.grp,self.x_val,self.y_val,self.z_val=createLCS()
		
		self.form.addsplitpoint.clicked.connect(self.addsplitpt)
		self.form.removesplitpoint.clicked.connect(self.removesplitpt)
		self.form.createcsvfile.clicked.connect(self.createcsvfile) 
		self.form.createtable.clicked.connect(self.updatetable) 
		#self.form.listsplitpoint.itemClicked.connect(self.highlightlcs)
		self.form.sequencetable.setColumnCount(8)
		self.form.sequencetable.setColumnWidth(0,60)
		self.form.sequencetable.setHorizontalHeaderItem(0, QtGui.QTableWidgetItem('Start'))
		self.form.sequencetable.setColumnWidth(1,60)
		self.form.sequencetable.setHorizontalHeaderItem(1, QtGui.QTableWidgetItem('End'))
		self.form.sequencetable.setColumnWidth(2,80)
		self.form.sequencetable.setHorizontalHeaderItem(2, QtGui.QTableWidgetItem('Length'))
		#self.form.sequencetable.setColumnWidth(3,40)
		self.form.sequencetable.setHorizontalHeaderItem(3, QtGui.QTableWidgetItem('WPS'))
		#self.form.sequencetable.setColumnWidth(4,50)
		self.form.sequencetable.setHorizontalHeaderItem(4, QtGui.QTableWidgetItem('Position'))
		self.form.sequencetable.setColumnWidth(5,65)
		self.form.sequencetable.setHorizontalHeaderItem(5, QtGui.QTableWidgetItem('Reverse'))
		self.form.sequencetable.setColumnWidth(6,60)
		self.form.sequencetable.setHorizontalHeaderItem(6, QtGui.QTableWidgetItem('Seqnb'))
		self.form.sequencetable.setColumnWidth(7,60)
		self.form.sequencetable.setHorizontalHeaderItem(7, QtGui.QTableWidgetItem('Robnb'))
		self.form.sequencetable.setColumnWidth(6,60)
		
		self.choosenlcs=[]
		self.choosenidx=[]
		self.sequence=[]
		self.seqcirc=[]
		self.seq_combobox=[]
		self.form.scrollbarptnb.setRange(0,(len(obj.Pts)-1))
		self.circums=wc.sumVecLengths(obj.Pts,False)[0]
		

	def valuechange(self):
		#idx = self.form.Sliderptnb.value()
		idx = self.form.scrollbarptnb.value()
		self.form.currentpoint.setText(str(idx))
		circ=round(self.circums[idx],2)
		self.form.currentcircum.setText(str(circ))
		#print('nb: ',idx)
		lcs_pt=obj.Pts[idx]
		lcs_vecs=obj.Vecs[idx]
		updateLCS(self.grp,self.x_val,self.y_val,self.z_val,lcs_pt,lcs_vecs)
	
	
	def getStandardButtons(self):
		return (
			int(QtGui.QDialogButtonBox.Ok)
			| int(QtGui.QDialogButtonBox.Apply)
			| int(QtGui.QDialogButtonBox.Cancel)
		)
	
	def getPosition(self,index):
		self.form.position.clear()
		if list(self.cfg[self.form.wpsname.currentText()].keys()):
			#print(list(self.cfg[self.form.wpsname.currentText()].keys()))
			self.form.position.addItems ( list(self.cfg[self.form.wpsname.currentText()].keys()))
	
	
	def getPositionList(self,signalidx):
		#print ('signalidx: ',signalidx)
		#print ('wpscom: ',self.wpscom)
		for boxidx,wps in enumerate(self.wpscom):
			pos=self.poscom[boxidx]
			#if [wps.currentIndex(),pos.currentIndex()]!=self.comboidxs[boxidx]:
			if wps.currentIndex()!=self.comboidxs[boxidx][0]:
				#idx=wps.currentIndexChanged()
				#print('boxidx: ',boxidx,' | idx: ',idx)
				pos.clear()
				if list(self.cfg[wps.currentText()].keys()):
					#print(list(self.cfg[self.form.wpsname.currentText()].keys()))
					pos.addItems ( list(self.cfg[wps.currentText()].keys()))
				self.comboidxs[boxidx][0]=wps.currentIndex()
				self.comboidxs[boxidx][1]=pos.currentIndex()
	
	
	def clicked(self, button):
		#if button == int(QtGui.QDialogButtonBox.Apply):
		self.accept(button)
	
	def accept(self, signal=None):
		#a=self.form.wpsname.currentText()
		#b=self.form.position.currentText()
		#print (a,' | ',b)
		self.choosenlcs.append(self.grp)
		for lcs in self.choosenlcs:
			deleteLCS(lcs)
		
		FreeCADGui.Control.closeDialog()
	
	
	def getsequences(self):
		seq=[]
		circs=[]
		self.seqcirc=[]
		self.sequence=[]
		if len(self.choosenidx)>0:
			if obj.closed is True:
				modseq=self.choosenidx[:]+[self.choosenidx[0]]
			else:
				modseq=self.choosenidx[:]
				if modseq[0]!=0:
					modseq.insert(0,0)
				if modseq[-1]!=(len(obj.Pts)-1):
					modseq.append((len(obj.Pts)-1))			
			
			for i in range((len(modseq)-1)):
				startidx=modseq[i]
				endidx=modseq[i+1]
				self.sequence.append([startidx,endidx])
				if endidx<=startidx: #extends over zero, is a closed wire
					d0=-(self.circums[-1]-self.circums[startidx])
					d1=self.circums[endidx]
				else:
					d0=self.circums[startidx]
					d1=self.circums[endidx]
				self.seqcirc.append(round((d1-d0),2))
			print ('Sequences: ',self.sequence)
			#print('circs: ',circs)
	
	
	def addsplitpt(self):
		#ptidx=self.form.Sliderptnb.value()
		ptidx=self.form.scrollbarptnb.value()
		if self.choosenidx.count(ptidx)==0:
			self.form.listsplitpoint.addItem( str(ptidx))
			grp,x_val,y_val,z_val=createLCS()
			lcs_pt=obj.Pts[ptidx]
			lcs_vecs=obj.Vecs[ptidx]
			updateLCS(grp,x_val,y_val,z_val,lcs_pt,lcs_vecs)
			self.choosenlcs.append(grp)
			self.choosenidx.append(ptidx)
			self.choosenidx.sort()
			self.getsequences()
	
	
	def sequpdate(self,signalidx):
		#print ('seq_signalidx: ',signalidx)
		#print ('wpscom: ',self.seqcom)
		for boxidx,seq in enumerate(self.seqcom):
			choosenidx=seq.currentIndex()
			if choosenidx!=self.seqcomboidxs[boxidx]:
				#print ('choosenidx: ',choosenidx)
				#print ('seqcomboidxs: ',self.seqcomboidxs)
				changeidx=self.seqcomboidxs.index(choosenidx)
				#print ('changeidx: ',changeidx)
				new_val=self.seqcomboidxs[boxidx]
				self.seqcomboidxs[changeidx]=new_val #exchange the values
				self.seqcomboidxs[boxidx]=seq.currentIndex()
				self.seqcom[changeidx].setCurrentIndex(new_val) #exchange the values ?triggers a run on the new value wit sequpdate due to seqcomboBox.currentIndexChanged
	
	def updatetable(self):
		row=0
		self.form.sequencetable.clearContents ()
		#self.form.sequencetable.setRowCount(len(self.choosenidx))
		seqnb=[str(i) for i in range(len(self.sequence))] #for seqcombo choice
		robnb=[str(i) for i in range(1,self.form.robotqty.value()+1,1)] #for seqcombo choice
		self.seqcom=[]
		self.wpscom=[]
		self.poscom=[]
		self.comboidxs=[]
		self.seqcomboidxs=[]
		for seqidx,elem in enumerate(self.sequence):
			#print ('elem: ',elem)
			self.form.sequencetable.insertRow(row)
			#self.form.sequencetable.setItem(row, 0, QtGui.QTableWidgetItem(str(elem)))
			self.form.sequencetable.setItem(row, 0, QtGui.QTableWidgetItem(str(elem[0])))
			self.form.sequencetable.setItem(row, 1, QtGui.QTableWidgetItem(str(elem[1])))
			self.form.sequencetable.setItem(row, 2, QtGui.QTableWidgetItem(str(self.seqcirc[seqidx])))
			wpscomboBox = QtGui.QComboBox()
			
			wpscomboBox.addItems(list(self.cfg.keys()))
			self.wpscom.append(wpscomboBox)
				
			poscomboBox = QtGui.QComboBox()
			self.poscom.append(poscomboBox)
			#self.getPositionListOLD(self.wpscom[seqidx],self.poscom[seqidx])
			self.comboidxs.append([-1,-1])
			self.getPositionList(0)
			#self.comboidxs.append([wpscomboBox.currentIndex(),poscomboBox.currentIndex()])
			
			self.form.sequencetable.setCellWidget(row, 3, wpscomboBox)
			self.form.sequencetable.setCellWidget(row, 4, poscomboBox)
			#wpscomboBox.currentIndexChanged.connect(self.getPositionList(poscomboBox))
			#self.windowTitleChanged.connect(lambda x: self.my_custom_fn(x, 25))
			#wpscomboBox.currentIndexChanged.connect(lambda idxchg: self.getPositionList(wpscomboBox,poscomboBox))
			#self.wpscom[seqidx].currentIndexChanged.connect(lambda idxchg: self.getPositionListOLD(self.wpscom[seqidx],self.poscom[seqidx]))
			self.wpscom[seqidx].currentIndexChanged.connect(self.getPositionList)
						
			revCheck = QtGui.QCheckBox()
			revCheck.setCheckState(QtCore.Qt.Unchecked) 
			self.form.sequencetable.setCellWidget(row, 5, revCheck)
			
			seqcomboBox = QtGui.QComboBox()			
			seqcomboBox.addItems(seqnb)
			
			seqcomboBox.activated.connect(self.sequpdate)
			self.form.sequencetable.setCellWidget(row, 6, seqcomboBox)
			seqcomboBox.currentIndexChanged.connect(self.sequpdate)
			self.seqcom.append(seqcomboBox)
			self.seqcomboidxs.append(seqidx)
			seqcomboBox.setCurrentIndex(seqidx)
			
			qtycomboBox = QtGui.QComboBox()			
			qtycomboBox.addItems(robnb)
			self.form.sequencetable.setCellWidget(row, 7, qtycomboBox)
			qtycomboBox.setCurrentIndex(0)
			#revCheck.setTextAlignment(QtCore.Qt.AlignHCenter) # change the alignment
			
			row=row+1
		self.form.sequencetable.setRowCount(len(self.sequence)) #len(self.choosenidx)
		self.actualidx=None
	
	
	def removesplitpt(self):
		idx=self.form.listsplitpoint.currentRow()
		if idx>=0:
			curr_row=self.form.listsplitpoint.currentRow()
			self.choosenidx.remove(int(self.form.listsplitpoint.item(curr_row).text()))	
			self.form.listsplitpoint.takeItem(curr_row)	
			#print('lcs: ',self.choosenlcs)
			deleteLCS(self.choosenlcs.pop(idx))
			#print('lcs after: ',self.choosenlcs)	
			self.getsequences()
			#self.updatetable()
	
	def createcsvfile(self):
		qtyrow=self.form.sequencetable.rowCount()
		if qtyrow>0:
			seqtext=[[] for i in range(len(obj.Pts))]
			wpstext=[[] for i in range(len(obj.Pts))]
			passorder=[[] for i in range(len(obj.Pts))]
			comptext=[]
			
			seqatrow=[self.form.sequencetable.cellWidget(rownb, 6).currentText() for rownb in range(qtyrow)]
			seqatrowsorted=seqatrow[:]
			seqatrowsorted.sort()
			rowsbyseq=[seqatrow.index(seq) for seq in seqatrowsorted] 
			#for rownb in range(qtyrow):
			for rownb in rowsbyseq: #go trough sorted seqences
				start=int(self.form.sequencetable.item(rownb,0).text())
				end=int(self.form.sequencetable.item(rownb,1).text())
				wps=self.form.sequencetable.cellWidget(rownb, 3).currentText()
				pos=self.form.sequencetable.cellWidget(rownb, 4).currentText()
				reverse=self.form.sequencetable.cellWidget(rownb,5).isChecked()
				seqnb=self.form.sequencetable.cellWidget(rownb, 6).currentText()
				robnb=self.form.sequencetable.cellWidget(rownb, 7).currentText()
				#alternate=self.form.sequencetable.cellWidget(rownb,8).isChecked()
				#positiveside=self.form.sequencetable.cellWidget(rownb,9).isChecked()
				alternate=self.form.alternate.isChecked()
				positiveside=self.form.startpositive.isChecked()
				passordernb=(alternate<<1) | positiveside #bitwise or binary representation
				if reverse is True:
					dirtext='r'
				else:
					dirtext='f'
				if obj.closed is True:
					shapetext='c'
				else:
					shapetext='o'			
				
				x=seqnb+'_'+robnb+dirtext+shapetext
				print (start,' ',end,' ',x)
				if start>=end: #implicit closed
					first=[i for i in range(start,len(obj.Pts),1)]
					second=[i for i in range(0,end+1,1)]
					seqrange=first+second
				else:
					seqrange=[i for i in range(start,end+1,1)]				
				
				for seqnb in seqrange:
					if len(seqtext[seqnb])==0: #first entry
						seqtext[seqnb]=[x]					
					else:
						seqtext[seqnb][0]=seqtext[seqnb][0]+' '+x
					if len(wpstext[seqnb])==0: #first entry
						wpstext[seqnb].append(wps)
						wpstext[seqnb].append(pos)
						passorder[seqnb].append(passordernb)

				'''
				#old approach has problems with only one sequence writing the start/end == 2 sequences at one point
				for linenb,linetext in enumerate(seqtext):
					if linenb in seqrange:
						if len(seqtext[linenb])==0: #first entry
							seqtext[linenb]=[x]
						else:
							seqtext[linenb][0]=seqtext[linenb][0]+' '+x
						#wps writing
						if len(wpstext[linenb])==0: #first entry
							wpstext[linenb].append(wps)
							wpstext[linenb].append(pos)
							passorder[linenb].append(passordernb)
				'''
			for i,s in enumerate(seqtext):
				#print(s,' | ',wpstext[i])
				comptext.append(seqtext[i]+wpstext[i]+passorder[i])
				print(comptext[-1])
			filepath=FreeCAD.ActiveDocument.FileName
			directory=os.path.dirname(filepath)
			SaveName, Filter = QtGui.QFileDialog.getSaveFileName(None, "Save .csv", directory,             "*.csv") # PySide
			fileabs,fileextension=os.path.splitext(SaveName)
			#fileabs="/home/jb/data/slicecoords.csv"
			fileabs=fileabs+'.csv'
			STLfuncs.slicerCSV(obj.Pts,obj.Vecs,comptext,fileabs)


##############################################################################
##############################################################################
 
class ConnectedFacesTaskPanel:
	def __init__(self):
		# this will create a Qt widget from our ui file
		path_to_ui =os.path.join(basefolder,"connectedfaces.ui")
		self.form = FreeCADGui.PySideUic.loadUi(path_to_ui)
		self.form.weld.clicked.connect(self.getWeldCompound)
		self.form.adjacent.clicked.connect(self.getCompound)
		self.connectedCompounds=None
		self.weldCompound=None
		self.connectedFaces=None
		
	def getCompound(self):
		self.adjacentCompounds=fromSelection()[0]
	
	def getWeldCompound(self):
		self.weldCompound=Part.Compound(fromSelection()[0])
	
	
	def accept(self):
		if self.adjacentCompounds is not None and self.weldCompound is not None:
			self.connectedFaces=JoinedFaces(self.weldCompound,self.adjacentCompounds)
			#facecomp=Part.Compound(self.connectedFaces)
			facelist=[]
			for f in self.connectedFaces:
				facelist.extend(f.Faces)
			facecomp=Part.makeShell(facelist)			
			
			color=(170/255.,170/255.,127/255.)
			myObject=FreeCAD.ActiveDocument.addObject("Part::Feature","connectedFaces")
			myObject.ViewObject.LineColor=color
			myObject.Shape = facecomp
			
			filepath=FreeCAD.ActiveDocument.FileName
			directory=os.path.dirname(filepath)
			SaveName, Filter = QtGui.QFileDialog.getSaveFileName(None, "Save .stl", directory,             "*.stl") # PySide
			fileabs,fileextension=os.path.splitext(SaveName)
			fileabs=fileabs+'.stl'
			
			facecomp.exportStl(fileabs)
			
		FreeCADGui.Control.closeDialog()


##############################################################################
##############################################################################

class WeldCoverFacesTaskPanel:
	def __init__(self):
		# this will create a Qt widget from our ui file
		path_to_ui =os.path.join(basefolder,"weldcoverfaces.ui")
		self.form = FreeCADGui.PySideUic.loadUi(path_to_ui)
		self.form.coverfaces.clicked.connect(self.getWeldCoverCompound)
		self.weldcoverCompound=None
		
	def getWeldCoverCompound(self):
		self.weldcoverCompound=Part.Compound(fromSelection()[1])
		print('weldcover: ',self.weldcoverCompound)
	
	
	def accept(self):
		if self.weldcoverCompound is not None:
			
			color=(170/255.,170/255.,127/255.)
			myObject=FreeCAD.ActiveDocument.addObject("Part::Feature","WeldCoverFaces")
			myObject.ViewObject.LineColor=color
			myObject.Shape = self.weldcoverCompound
			
			filepath=FreeCAD.ActiveDocument.FileName
			directory=os.path.dirname(filepath)
			SaveName, Filter = QtGui.QFileDialog.getSaveFileName(None, "Save .stl", directory,             "*.stl") # PySide
			fileabs,fileextension=os.path.splitext(SaveName)
			fileabs=fileabs+'.stl'
			
			self.weldcoverCompound.exportStl(fileabs)
			
		FreeCADGui.Control.closeDialog()


##############################################################################
##############################################################################


class SimTaskPanel:
	def __init__(self):
		# this will create a Qt widget from our ui file
		path_to_ui =os.path.join(basefolder,"sim.ui")
		self.form = FreeCADGui.PySideUic.loadUi(path_to_ui)
		
		self.tooldocobj=[]
		self.places=[]
		self.refplace=None

		#self.load() #DEBUG
		self.form.loadfiles.clicked.connect(self.load)

		self.form.selecttool.clicked.connect(self.get_tool)

		self.form.simspeedslider.setRange(1,100)
		self.form.simspeedval.setText(str(1))
		self.form.simspeedslider.valueChanged.connect(self.simspeedupdate) #sliderReleased

		self.form.posnbslider.valueChanged.connect(self.posnbupdate) #sliderReleased
		self.form.play.toggled.connect(self.timeronoff)
		self.form.reset.clicked.connect(self.refhome)

		self.timer = QtCore.QTimer()# a timer
		#Connect the timeout signal with my update method.
		QtCore.QObject.connect(self.timer, QtCore.SIGNAL("timeout()"), self.simulate)
		#self.timer.start(50)

		self.form.currposnb.textChanged.connect(self.toolupdate) #performs the update whenever text is changed
	
	def timeronoff(self):
		if self.form.play.isChecked() is True: 
			self.timer.start(50)
		else:
			self.timer.stop()


	def simspeedupdate(self):
		val = self.form.simspeedslider.value()
		self.form.simspeedval.setText(str(val))
	

	def get_tool(self):
		self.tooldocobj=(fromSelection()[2][0])
		self.refplace=self.tooldocobj.Placement
		self.form.currenttool.setText(self.tooldocobj.Label)
		print('Tool document obj: ',self.tooldocobj)
	

	def posnbupdate(self):
		#idx = self.form.Sliderptnb.value()
		idx = self.form.posnbslider.value()
		self.form.currposnb.setText(str(idx))
		self.form.posnbslider.setValue(idx)


	def toolupdate(self):
		#if self.form.play.isChecked() is True:
		idx=int(self.form.currposnb.text())
		print('toolupdate: ',idx)
		self.form.posnbslider.setValue(idx)
		if self.refplace is not None:
			STLvis.simpath(self.tooldocobj,self.refplace,self.places,idx)


	def simulate(self):
		if self.form.play.isChecked() is True and self.refplace is not None:
			print('play recognized')
			inc=int(self.form.simspeedval.text())
			newval=int(self.form.currposnb.text())+inc
			print('newval: ',newval)
			if newval<len(self.places):
				print('try to update')
				self.form.currposnb.setText(str(newval))
			else:
				self.form.play.setChecked(False)


	def refhome(self):
		#if self.form.play.isChecked() is False:
		self.form.play.setChecked(False)
		print('refhome activated')
		idx=0
		self.form.currposnb.setText(str(idx))
		#self.toolupdate()
		self.tooldocobj.Placement=self.refplace


	def load(self):
		'''		
		my_dir = ""
		filepath=FreeCAD.ActiveDocument.FileName
		directory=os.path.dirname(filepath)
		my_dir = QtGui.QFileDialog.getExistingDirectory(FreeCADGui.getMainWindow(),"Open a folder",directory,QtGui.QFileDialog.ShowDirsOnly)
		'''
		my_dir = loaddialog()
		#my_dir='/home/jb/data/Client_2/Weld_3/Version_0/Pattern_0/Scan_0/Layer_0/'
		if my_dir is not "":
			my_dir=my_dir+'/'
			#my_dir='/home/jb/data/Client_2/Weld_3/Version_0/Pattern_0/Scan_0/Layer_0/'
			gendata=STLvis.SmlData()
			gendata.currLayer=my_dir  
			blockelem=STLvis.extractpaths(gendata)
			showrapid=True #False
			showseg=True #False
			self.places=STLvis.createpath(blockelem,showrapid,showseg)
		self.form.dirfile.setText(str(my_dir))
		self.form.currposnb.setText(str(0))
		self.form.maxposnb.setText(str(len(self.places)-1))
		self.form.posnbslider.setRange(0,(len(self.places)-1))

	def reject(self):
		self.accept()

	def accept(self):
		if self.refplace is not None:
			self.timer.stop()
			self.refhome()
		FreeCADGui.Control.closeDialog()


##############################################################################
##############################################################################


class showPathsTaskPanel:
	def __init__(self):
		# this will create a Qt widget from our ui file
		#path_to_ui =os.path.join(basefolder,"showPaths.ui")
		#self.form = FreeCADGui.PySideUic.loadUi(path_to_ui)
		pass
			
	def load(self):
		'''		
		my_dir = ""
		filepath=FreeCAD.ActiveDocument.FileName
		directory=os.path.dirname(filepath)
		my_dir = QtGui.QFileDialog.getExistingDirectory(FreeCADGui.getMainWindow(),"Open a folder",directory,QtGui.QFileDialog.ShowDirsOnly)
		'''
		my_dir = loaddialog()
		if my_dir is not "":
			my_dir=my_dir+'/'
			gendata=STLvis.SmlData()
			gendata.currLayer=my_dir  
			STLvis.showPaths(gendata)
			

##############################################################################
##############################################################################

def loaddialog():		
	my_dir = ""
	filepath=FreeCAD.ActiveDocument.FileName
	directory=os.path.dirname(filepath)
	my_dir = QtGui.QFileDialog.getExistingDirectory(FreeCADGui.getMainWindow(),"Open a folder",directory,QtGui.QFileDialog.ShowDirsOnly)
	return my_dir

##############################################################################
##############################################################################  

def fromSelection():
	sf = FreeCADGui.Selection.getSelectionEx()
	#print(sf)
	elx=[]
	bodyelx=[]
	objlist=[]
	for sel in sf:
		objlist.append(sel.Object)
		bodyelx.append(sel.Object.Shape)
		print('sel: ',sel)
		if sel.HasSubObjects is True:
			selelm=sel.SubObjects
			elx.extend(selelm)
		else:
			selelm=sel.Object.Shape
			elx.append(selelm)
	print('Subelementselection: ',elx)
	print('Bodyselection: ',bodyelx)
	FreeCADGui.Selection.clearSelection()
	return bodyelx,elx,objlist


##############################################################################
##############################################################################

def JoinedFaces(weld,complist):
	connected=[]
	for comp in complist:
		connected.append([])
		for face in comp.Faces:
			dist=face.distToShape(weld)[0]
			print('dist: ',dist)
			if dist<1e-3:
				connected[-1].append(face)
		connected[-1]=Part.Compound(connected[-1])
	return connected

##############################################################################
##############################################################################

def createLCS():
	grp=coin.SoSeparator()
	
	x_mat = coin.SoMaterial()
	y_mat = coin.SoMaterial()
	z_mat = coin.SoMaterial()
	
	x_width=coin.SoDrawStyle()
	x_width.lineWidth.setValue=2
	
	y_width=coin.SoDrawStyle()
	y_width.lineWidth.setValue=2
	
	z_width=coin.SoDrawStyle()
	z_width.lineWidth.setValue=2
	
	#mat.ambientColor = (0.6666666, 1.0, 0)  
	x_mat.diffuseColor = (0, 0, 1.0) #blue   
	y_mat.diffuseColor = (1.0, 0, 0) #red	
	z_mat.diffuseColor = (0, 1.0, 0) #green
	
	
	x_val=coin.SoCoordinate3()
	x_lin=coin.SoLineSet()
	
	y_val=coin.SoCoordinate3()
	y_lin=coin.SoLineSet()
	
	z_val=coin.SoCoordinate3()
	z_lin=coin.SoLineSet()
	#x_val.point.setValues([coin.SbVec3f(0.,0.,0.), coin.SbVec3f(10.,10.,10.)])
	grp.addChild(x_mat)
	grp.addChild(x_width)
	grp.addChild(x_val)
	grp.addChild(x_lin)	
	
	grp.addChild(y_mat)
	grp.addChild(y_width)
	grp.addChild(y_val)
	grp.addChild(y_lin)	
	
	grp.addChild(z_mat)
	grp.addChild(z_width)
	grp.addChild(z_val)
	grp.addChild(z_lin)	
	
	FreeCADGui.ActiveDocument.ActiveView.getSceneGraph().addChild(grp)
	return grp,x_val,y_val,z_val

##############################################################################
##############################################################################

def updateLCS(grp,x_val,y_val,z_val,lcs_pt,lcs_vecs):
	'''
	update the lcs position
	'''
	#x_val.point.setValues([coin.SbVec3f(0.,0.,0.), coin.SbVec3f(10.,10.,10.)])
	baselength=10
	for i,val in enumerate([x_val,y_val,z_val]):
		end=lcs_pt+(lcs_vecs[i]*(i+1)*baselength)
		val.point.setValues([coin.SbVec3f(lcs_pt.x,lcs_pt.y,lcs_pt.z), coin.SbVec3f(end.x,end.y,end.z)])
	return None

##############################################################################
##############################################################################

def deleteLCS(grp):
	return FreeCADGui.ActiveDocument.ActiveView.getSceneGraph().removeChild(grp)

##############################################################################
##############################################################################

#panelface = FacePanel()
#FreeCADGui.Control.showDialog(panelface)

global obj
obj=seclist()

FreeCADGui.addCommand('Face Panel',CmdFacePanel())
FreeCADGui.addCommand('Wire Task Panel',CmdWireTaskPanel())
FreeCADGui.addCommand('Point Task Panel',CmdPointTaskPanel())
FreeCADGui.addCommand('Wps Task Panel',CmdWpsTaskPanel())
FreeCADGui.addCommand('Connected Faces Task Panel',CmdConnectedFacesTaskPanel())
FreeCADGui.addCommand('Weld Cover Faces Task Panel',CmdWeldCoverFacesTaskPanel())
FreeCADGui.addCommand('Show Paths Task Panel',CmdshowPathsTaskPanel())
FreeCADGui.addCommand('Simulate Paths Task Panel',CmdsimTaskPanel())
