# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------
# Name:        module1
# Purpose:
#
# Author:      Johannes Bichler 
#
# Created:     08.08.2021
# Copyright:   (c) Johannes Bichler 2021
# Licence:     <your licence>
#-------------------------------------------------------------------------------

import sys
from FreeCAD import Console
import Part
import time
import numpy as np


################################################################################
################################################################################

def sumVecLengths(veclist,targetlength):
	'''
	get the circums of a wire represented py vectorlist
	return circumlist,vectorindexpairs,vectorpairs
	'''
	circum=[]
	pair=[]
	indexpair=[]
	curr_circum=0
	lastvec=None
	lastvecidx=None
	for vecidx,vec in enumerate(veclist):
		if lastvec:
			length=(vec-lastvec).Length
			pair.append([lastvec,vec])
			indexpair.append([lastvecidx,vecidx])
			curr_circum=curr_circum+length
			circum.append(curr_circum)
		else:
			pair.append([lastvec,vec])
			indexpair.append([lastvecidx,vecidx])
			circum.append(curr_circum)

		lastvec=vec
		lastvecidx=vecidx
		if targetlength is not False and targetlength<=curr_circum:
			break
	return circum,indexpair,pair


################################################################################
################################################################################

def find_edge_from_vertexes(edgelist,vertex_list):
	'''
		find edge in edgelist by end vertexlist
		find edge in edgelist by 1 endpoint -can be list or vertex
		can only be one Part.edge
	input: edgelist, vertex_list
	return: edge,index in edgelist
	'''
	edge_ID=False
	index=False
	if edgelist.__class__==Part.Edge:
		inter=edgelist
		edgelist=[]
		edgelist.append(inter)
	if vertex_list.__class__==Part.Vertex:
		inter=vertex_list
		vertex_list=[]
		vertex_list.append(inter)
	for i in range(len(edgelist)):
		count=0
		for j in range(len(edgelist[i].Vertexes)):
			origin_vertex=edgelist[i].Vertexes[j]
			for k in range(len(vertex_list)):
				target_vertex=vertex_list[k]
				if target_vertex.Point.distanceToPoint(origin_vertex.Point)<0.001:
					count=count+1
				if count==2:
					edge_ID=edgelist[i]
					index=i
			#optinal find edge=end of wire by 1 point
			if (len(vertex_list))==1:
				target_vertex=vertex_list[k]
				if target_vertex.Point.distanceToPoint(origin_vertex.Point)<0.001:
					edge_ID=edgelist[i]
					index=i
	return edge_ID,index

################################################################################
################################################################################

def find_endpoints_h(top_wire):
	'''
	find endpoints of wire.Edges = connection to joining parts
	input: top_wire
	return: LIST (endpoints::Vertex object)
	'''
	endpoints=[]
	all_vertexes=[]
	for edge in top_wire:
		all_vertexes.extend([edge.Vertexes[0].hashCode(),edge.Vertexes[1].hashCode()])	#TODO check what circle delivers
	
	np_all_vertexes=np.asarray(all_vertexes)
	#np_unique_vh,unique_indices,unique_inverse,counts=np.unique(np_all_vertexes,return_index=False, return_inverse=False, return_counts=True, axis=None)
	np_unique_vh,counts=np.unique(np_all_vertexes,return_index=False, return_inverse=False, return_counts=True, axis=None)
	idx_into_unique=np.where(counts==1)[0]
	np_unique_vh=np.take(np_unique_vh,idx_into_unique)
	singlevertidx=[np.where(np_all_vertexes==vh)[0][0] for vh in np_unique_vh]

	#singlevertidx=[index for index,vh in enumerate(all_vertexes) if np.where(np_all_vertexes==vh)[0].shape[0]==1]

	for index in singlevertidx:
			edge_nb=index//2
			v_nb=index%2
			endpoints.append(top_wire[edge_nb].Vertexes[v_nb])

	if len(endpoints)==0:
		endpoints=False
		#raise TypeError('No unique Endpoints')
	return endpoints

################################################################################
################################################################################

def find_wire_end_and_tangent_c(edgelist,endpoints):
	'''
		find from wire-endpoints  related edge and tangent pointing outwards
		can be also only one vertex
	input: edge_list,endpoints_list
	return: list of [endpoint,tangent]
	'''
	result=[]
	if endpoints.__class__==Part.Vertex:
		inter=endpoints
		endpoints=[]
		endpoints.append(inter)

	for edge in edgelist:
		for vertex in edge.Vertexes:
			for endpoint in endpoints:
				if vertex.Point.distanceToPoint(endpoint.Point)<0.001:
					params=edge.ParameterRange
					if edge.valueAt(params[0]).distanceToPoint(endpoint.Point)<0.001:
						tangent=edge.tangentAt(params[0])
						tangent=tangent.negative()
					else:
						tangent=edge.tangentAt(params[1])
					result.append(vertex)
					result.append(tangent)
	if len(result)==0:
		result=False
		#raise TypeError('No tangent found')
	return result

################################################################################
################################################################################

def check_point_on_edge_weave(base_wire,poi_vector):
	'''
	check if point is on edge
	wire::list
	poi_vector::vector
	return edge,index,parameter
	'''
	start_processing=time.time()
	poi=poi_vector
	tol=1e-5 #was 1e-1
	dist_tol=1e-5	#5e-2
	quality=tol
	if base_wire.__class__==Part.Edge:
		base_wire=[base_wire]
	else:
		base_wire=base_wire.Edges

	index=False
	final_param=False
	edge=False
	dist_to_wire=False
	Console.PrintMessage("__________________________________________________________________"+'\n')
	#Console.PrintMessage("Check point on edge number elements:  " + str(len(base_wire))+'\n')
	dist_solution=[]
	final_param_list=[]
	for i in range(len(base_wire)):
		'''
		first_param=base_wire[i].FirstParameter
		last_param=base_wire[i].LastParameter
		Console.PrintMessage("Check point on edge first|last parameter:  " + str(first_param)+' | '+str(last_param)+'\n')
		try:
			param=base_wire[i].parameterAt(Part.Vertex(poi))
			Console.PrintMessage("Check point on edge parameter:  " + str(param)+'\n')
			poi_param=base_wire[i].valueAt(param)
			dist=poi_param.distanceToPoint(poi)
			if dist<=tol:
				final_param=param
				tol=dist
				index=i
		except:
		'''

		dist_to_wire_solution=base_wire[i].distToShape(Part.Vertex(poi))
		dist_to_wire=dist_to_wire_solution[0]
		#Console.PrintMessage("Check point on edge dist to wire:  " + str(dist_to_wire_solution)+'\n')
		dist_solution.append(dist_to_wire)
		final_param_list.append(dist_to_wire_solution)


	i=dist_solution.index(min(dist_solution))
	dist_to_wire=min(dist_solution)
	dist_to_wire_solution=final_param_list[i]
	Console.PrintMessage("Choosen edge:  " + str(i)+'\n')
	Console.PrintMessage("Nearest distance:  " + str(min(dist_solution))+'\n')
	Console.PrintMessage("Distance list:  " + str(dist_solution)+'\n')

	if dist_to_wire<=dist_tol:
		dist_tol=dist_to_wire
		first_param=base_wire[i].FirstParameter
		last_param=base_wire[i].LastParameter
		#edge_param=base_wire[i].parameterAt(Part.Vertex(poi))
		Console.PrintMessage("Check point on edge dist to wire:  " + str(dist_to_wire_solution)+'\n')
		#index=i
		#for j in range(len(base_wire[i].Vertexes)):
		#vertex_point=base_wire[i].Vertexes[j].Point
		first_param_dist=base_wire[i].valueAt(first_param).distanceToPoint(poi)
		last_param_dist=base_wire[i].valueAt(last_param).distanceToPoint(poi)

		if first_param_dist<=tol:
			final_param=first_param
			index=i
			tol=first_param_dist
			Console.PrintMessage("Check point on edge first param:  " + str(final_param)+'\n')
		elif last_param_dist<=tol:
			final_param=last_param
			index=i
			tol=last_param_dist
			Console.PrintMessage("Check point on edge last param:  " + str(final_param)+'\n')
		elif first_param_dist>tol and last_param_dist>tol and final_param is False: #if some intermediate parameter
			Console.PrintMessage("Check point on edge dist to wire tolerance:  " + str(dist_tol)+'\n')
			Console.PrintMessage("Check point on edge dist to wire:  " + str(dist_to_wire_solution[2])+'\n')
			final_param=dist_to_wire_solution[2][0][2] #param on edge
			index=i

	if final_param is not False:
		Console.PrintMessage("Check point on edge tolerance tol|first dist|last dist:  " + str(tol)+' | '+str(round(first_param_dist,5))+' | '+str(round(last_param_dist,5))+'\n')
		Console.PrintMessage("Check point on edge tolerance first param|current param|last param:  " + str(round(first_param,3))+' | '+str(round(final_param,3))+' | '+str(round(last_param,3))+'\n')
	if index is not False:
		edge=base_wire[index]
	end_processing=time.time()
	Console.PrintMessage("Check point on edge time:  " + str(round(end_processing-start_processing,3)) + " [sec]"+'\n')
	Console.PrintMessage("__________________________________________________________________"+'\n')

	return edge,index,final_param

################################################################################
################################################################################

def main():
	pass

if __name__ == '__main__':
	main()


